@@ -30,9 +30,9 @@
 #ifndef __GEDIT_PREFS_MANAGER_H__
 #define __GEDIT_PREFS_MANAGER_H__
 
-#include <gdk/gdkcolor.h>
 #include <gtk/gtkenums.h>
 #include <glib/gslist.h>
+#include <gtksourceview/gtksourceview.h>
 #include "gedit-app.h"
 
 #define GEDIT_BASE_KEY	"/apps/gedit-2"
@@ -45,13 +45,6 @@
 #define GPM_EDITOR_FONT			GPM_FONT_DIR "/editor_font"
 #define GPM_SYSTEM_FONT			"/desktop/gnome/interface/monospace_font_name"
 
-#define GPM_COLORS_DIR			GPM_PREFS_DIR  "/editor/colors"
-#define GPM_USE_DEFAULT_COLORS		GPM_COLORS_DIR "/use_default_colors"
-#define GPM_BACKGROUND_COLOR		GPM_COLORS_DIR "/background_color"
-#define GPM_TEXT_COLOR			GPM_COLORS_DIR "/text_color"
-#define GPM_SELECTED_TEXT_COLOR		GPM_COLORS_DIR "/selected_text_color"
-#define GPM_SELECTION_COLOR		GPM_COLORS_DIR "/selection_color"
-
 #define GPM_SAVE_DIR			GPM_PREFS_DIR  "/editor/save"
 #define GPM_CREATE_BACKUP_COPY  	GPM_SAVE_DIR "/create_backup_copy"
 #define GPM_BACKUP_COPY_EXTENSION	GPM_SAVE_DIR "/backup_copy_extension"
@@ -85,14 +78,16 @@
 #define GPM_DISPLAY_RIGHT_MARGIN	GPM_RIGHT_MARGIN_DIR "/display_right_margin"
 #define GPM_RIGHT_MARGIN_POSITION	GPM_RIGHT_MARGIN_DIR "/right_margin_position"
 
+#define GPM_SMART_HOME_END_DIR		GPM_PREFS_DIR "/editor/smart_home_end"
+#define GPM_SMART_HOME_END		GPM_SMART_HOME_END_DIR "/smart_home_end"
+
 #define GPM_CURSOR_POSITION_DIR		GPM_PREFS_DIR  "/editor/cursor_position"
 #define GPM_RESTORE_CURSOR_POSITION	GPM_CURSOR_POSITION_DIR "/restore_cursor_position"
 
 #define GPM_SEARCH_HIGHLIGHTING_DIR	GPM_PREFS_DIR "/editor/search_highlighting"
 #define GPM_SEARCH_HIGHLIGHTING_ENABLE	GPM_SEARCH_HIGHLIGHTING_DIR "/enable"
 
-// FIXME: Still have to add schemas
-#define GPM_SOURCE_STYLE_DIR		GPM_PREFS_DIR "/editor/source_style"
+#define GPM_SOURCE_STYLE_DIR		GPM_PREFS_DIR "/editor/colors"
 #define GPM_SOURCE_STYLE_SCHEME		GPM_SOURCE_STYLE_DIR "/scheme"
 
 /* UI */
@@ -120,24 +115,9 @@
 #define GPM_PRINT_LINE_NUMBERS		GPM_PRINT_PAGE_DIR "/print_line_numbers"
 
 #define GPM_PRINT_FONT_DIR		GPM_PREFS_DIR "/print/fonts"
-#define GPM_PRINT_FONT_BODY		GPM_PRINT_FONT_DIR "/print_font_body"
-#define GPM_PRINT_FONT_BODY_PANGO	GPM_PRINT_FONT_DIR "/print_font_body_pango"
-#define GPM_PRINT_FONT_HEADER		GPM_PRINT_FONT_DIR "/print_font_header"
-#define GPM_PRINT_FONT_HEADER_PANGO	GPM_PRINT_FONT_DIR "/print_font_header_pango"
-#define GPM_PRINT_FONT_NUMBERS		GPM_PRINT_FONT_DIR "/print_font_numbers"
-#define GPM_PRINT_FONT_NUMBERS_PANGO	GPM_PRINT_FONT_DIR "/print_font_numbers_pango"
-
-#define GPM_WINDOW_DIR			"/gedit-2/window"
-#define GPM_WINDOW_STATE		GPM_WINDOW_DIR "/state"
-#define GPM_WINDOW_WIDTH		GPM_WINDOW_DIR "/width"
-#define GPM_WINDOW_HEIGHT		GPM_WINDOW_DIR "/height"
-#define GPM_SIDE_PANEL_SIZE		GPM_WINDOW_DIR "/side_panel_size"
-#define GPM_SIDE_PANEL_ACTIVE_PAGE	GPM_WINDOW_DIR "/side_panel_active_page"
-#define GPM_BOTTOM_PANEL_SIZE		GPM_WINDOW_DIR "/bottom_panel_size"
-#define GPM_BOTTOM_PANEL_ACTIVE_PAGE	GPM_WINDOW_DIR "/bottom_panel_active_page"
-
-#define GPM_FILE_FILTER			"/gedit-2/filefilter"
-#define GPM_ACTIVE_FILE_FILTER		GPM_FILE_FILTER "/id"
+#define GPM_PRINT_FONT_BODY		GPM_PRINT_FONT_DIR "/print_font_body_pango"
+#define GPM_PRINT_FONT_HEADER		GPM_PRINT_FONT_DIR "/print_font_header_pango"
+#define GPM_PRINT_FONT_NUMBERS		GPM_PRINT_FONT_DIR "/print_font_numbers_pango"
 
 /* Encodings */
 #define GPM_ENCODINGS_DIR		GPM_PREFS_DIR "/encodings"
@@ -164,12 +144,6 @@
 #define GPM_DEFAULT_EDITOR_FONT 	(const gchar*) "Monospace 12"
 #define GPM_DEFAULT_SYSTEM_FONT 	(const gchar*) "Monospace 10"
 
-#define GPM_DEFAULT_USE_DEFAULT_COLORS 	1 /* TRUE */
-#define GPM_DEFAULT_BACKGROUND_COLOR	(const gchar*) "#ffffffffffff"
-#define GPM_DEFAULT_TEXT_COLOR		(const gchar*) "#000000000000"
-#define GPM_DEFAULT_SELECTED_TEXT_COLOR	(const gchar*) "#ffffffffffff"
-#define GPM_DEFAULT_SELECTION_COLOR	(const gchar*) "#000000009c9c"
-
 #define GPM_DEFAULT_CREATE_BACKUP_COPY	1 /* TRUE */
 #define GPM_DEFAULT_BACKUP_COPY_EXTENSION (const gchar*) "~"
 
@@ -202,26 +176,12 @@
 #define GPM_DEFAULT_PRINT_WRAP_MODE	"GTK_WRAP_WORD"
 #define GPM_DEFAULT_PRINT_LINE_NUMBERS	0 /* No numbers */
 
-#define GPM_DEFAULT_PRINT_FONT_BODY_PANGO 	(const gchar*) "Monospace 9"
-#define GPM_DEFAULT_PRINT_FONT_HEADER_PANGO	(const gchar*) "Sans 11"
-#define GPM_DEFAULT_PRINT_FONT_NUMBERS_PANGO	(const gchar*) "Sans 8"
+#define GPM_DEFAULT_PRINT_FONT_BODY 	(const gchar*) "Monospace 9"
+#define GPM_DEFAULT_PRINT_FONT_HEADER	(const gchar*) "Sans 11"
+#define GPM_DEFAULT_PRINT_FONT_NUMBERS	(const gchar*) "Sans 8"
 
 #define GPM_DEFAULT_MAX_RECENTS		5
 
-#define GPM_DEFAULT_WINDOW_STATE	0
-#define GPM_DEFAULT_WINDOW_WIDTH	650
-#define GPM_DEFAULT_WINDOW_HEIGHT	500
-
-#define GPM_DEFAULT_WINDOW_STATE_STR	"0"
-#define GPM_DEFAULT_WINDOW_WIDTH_STR	"650"
-#define GPM_DEFAULT_WINDOW_HEIGHT_STR	"500"
-
-#define GPM_DEFAULT_SIDE_PANEL_SIZE	200
-#define GPM_DEFAULT_BOTTOM_PANEL_SIZE 	140
-
-#define GPM_DEFAULT_SIDE_PANEL_SIZE_STR   "200"
-#define GPM_DEFAULT_BOTTOM_PANEL_SIZE_STR "140"
-
 #define GPM_DEFAULT_HIGHLIGHT_CURRENT_LINE  1 /* TRUE */
 
 #define GPM_DEFAULT_BRACKET_MATCHING	  0 /* FALSE */
@@ -229,6 +189,8 @@
 #define GPM_DEFAULT_DISPLAY_RIGHT_MARGIN  0 /* FALSE */
 #define GPM_DEFAULT_RIGHT_MARGIN_POSITION 80
 
+#define GPM_DEFAULT_SMART_HOME_END	"AFTER"
+
 #define GPM_DEFAULT_SYNTAX_HL_ENABLE	1 /* TRUE */
 
 #define GPM_DEFAULT_WRITABLE_VFS_SCHEMES {"ssh", "sftp", "smb", "dav", "davs", NULL}
@@ -237,7 +199,7 @@
 
 #define GPM_DEFAULT_SEARCH_HIGHLIGHTING_ENABLE 1 /* TRUE */
 
-#define GPM_DEFAULT_SOURCE_STYLE_SCHEME "gvim"
+#define GPM_DEFAULT_SOURCE_STYLE_SCHEME "classic"
 
 typedef enum {
 	GEDIT_TOOLBAR_SYSTEM = 0,
@@ -269,31 +231,6 @@ gboolean		 gedit_prefs_manager_editor_font_can_set	(void);
 /* System font */
 gchar 			*gedit_prefs_manager_get_system_font		(void);
 
-/* Use default colors */
-gboolean 		 gedit_prefs_manager_get_use_default_colors 	(void);
-void			 gedit_prefs_manager_set_use_default_colors 	(gboolean udc);
-gboolean		 gedit_prefs_manager_use_default_colors_can_set	(void);
-
-/* Background color */	
-GdkColor		 gedit_prefs_manager_get_background_color	(void);
-void 			 gedit_prefs_manager_set_background_color	(GdkColor color);
-gboolean		 gedit_prefs_manager_background_color_can_set	(void);
-
-/* Text color */	
-GdkColor		 gedit_prefs_manager_get_text_color		(void);
-void 			 gedit_prefs_manager_set_text_color		(GdkColor color);
-gboolean		 gedit_prefs_manager_text_color_can_set		(void);
-
-/* Selection color */	
-GdkColor		 gedit_prefs_manager_get_selection_color	(void);
-void 			 gedit_prefs_manager_set_selection_color	(GdkColor color);
-gboolean		 gedit_prefs_manager_selection_color_can_set	(void);
-
-/* Selected text color */	
-GdkColor		 gedit_prefs_manager_get_selected_text_color	(void);
-void 			 gedit_prefs_manager_set_selected_text_color	(GdkColor color);
-gboolean		 gedit_prefs_manager_selected_text_color_can_set(void);
-
 /* Create backup copy */
 gboolean		 gedit_prefs_manager_get_create_backup_copy	(void);
 void			 gedit_prefs_manager_set_create_backup_copy	(gboolean cbc);
@@ -437,6 +374,12 @@ gint		 	 gedit_prefs_manager_get_right_margin_position	(void);
 void 			 gedit_prefs_manager_set_right_margin_position	(gint rmp);
 gboolean		 gedit_prefs_manager_right_margin_position_can_set (void);
 
+/* Smart home end */
+GtkSourceSmartHomeEndType
+		 	 gedit_prefs_manager_get_smart_home_end		(void);
+void 			 gedit_prefs_manager_set_smart_home_end		(GtkSourceSmartHomeEndType  smart_he);
+gboolean		 gedit_prefs_manager_smart_home_end_can_set	(void);
+
 /* Enable syntax highlighting */
 gboolean 		 gedit_prefs_manager_get_enable_syntax_highlighting (void);
 void			 gedit_prefs_manager_set_enable_syntax_highlighting (gboolean esh);
